/*
Compression example...
       
*/

#include <libps.h>
#include "pad.h"
#include "decomp.h"

// uncomment this for PAL screen mode (default NTSC)
//#define PAL_50HZ

#define SCREEN_WIDTH      512      
#define SCREEN_HEIGHT     240      
#define SCREEN_X_OFFSET     0
#define SCREEN_Y_OFFSET    16

#define WADADDR 0x8012A8BD  // Wad file
#define SPRITE_CNT 500
#define OT_LEN 1

GsOT		WorldOT[SPRITE_CNT];
GsOT_TAG	OTTags[SPRITE_CNT][1<<OT_LEN];
int		abuff;
PACKET	PA[2][SPRITE_CNT*sizeof(GsSPRITE)];
GsSPRITE	oursprite;
u_char  *Nemlogo;

// Variable declarations

volatile u_char *bb0, *bb1;

//  Routines that are defined:

void initsprite(u_long *imgloc);
static u_long PadRead();

int main(void)
{

  int     nobj = 1; 
  int     i, cnt, FileNo; 

// compile time switches
#ifdef PAL_50HZ
  SetVideoMode(MODE_PAL);
#else
  SetVideoMode(MODE_NTSC);
#endif

  GsInitGraph(SCREEN_WIDTH, SCREEN_HEIGHT, 4, 0, 0);

  GsDefDispBuff(0, 0, 0, 240);

  GetPadBuf(&bb0, &bb1);

  ResetGraph(0);

  for (i=0;i<SPRITE_CNT;i++)
  {
    	WorldOT[i].length=OT_LEN;
    	WorldOT[i].org=OTTags[i];
  }

  // We need to tell the decompress routines where to find our WAD/IPF file.
  // Simply pass the const address as a u_long pointer to WAD_InitSession and
  // your done..  From then on just use the other functions...  Check decomp.h
  // for routine information.
  WAD_InitSession((u_long *)(WADADDR));

  // uncomment to display the file information.
  //  printf("Extracting : %s \n",WAD_FileData(2));

  // For the demo I'm extracting, and displaying the NEMESIS logo image.
  // It happens to be file 2 in the list of files.  

  // Find the File number of the file we are looking to extract. 
  // Always use uppercase

  FileNo = WAD_FindName("NEMLGO2.TIM");

  // Get the filesize so we know how much memory we want to allocate.

  i =  WAD_FileSize(FileNo);

  // Malloc the memory...  Make sure you use a u_char pointer..  To allocate
  // 1 byte per char...  

  Nemlogo = (u_char *)malloc(i);

  // Extract the data for file #2 into our pointer Nemlogo.  All we do is 
  // pass the file # we want and the extract to pointer.  This routine
  // will return an int, if for some reason you want to know the size
  // of the expanded file.

  WAD_ExtractFile(FileNo, Nemlogo);      

  // Now pass our sprite to our display routine.

  initsprite((u_long *)Nemlogo);

  // We can now free the memory...  Since we have transfered the image
  // to the video buffer we no longer need to keep a copy of it.

  free(Nemlogo);
  
  do {   // Setting up the title screen, this will call game loop....
    abuff=GsGetActiveBuff();
    GsSetWorkBase((PACKET *)PA[abuff]);
    GsClearOt(0,0,&WorldOT[abuff]);
    GsSortFastSprite(&oursprite,&WorldOT[abuff],0);
	
    DrawSync(0);             

    cnt=VSync(0);          

    GsSwapDispBuff();        
    GsSortClear(0,0,0,&WorldOT[abuff]);
    GsDrawOt(&WorldOT[abuff]);	

    nobj = PadRead();

    // WOW you can move the logo around!!
    if (nobj & dPADLright) {oursprite.x+=3;}
    if (nobj & dPADLleft)  {oursprite.x-=3;}
    if (nobj & dPADLup)    {oursprite.y-=3;}
    if (nobj & dPADLdown)  {oursprite.y+=3;}
  
  } while ( !(nobj & dPADselect) );

        
  return(0);
}


void initsprite(u_long *imgloc)
{
	RECT 	rect;
	GsIMAGE	timdata;

  // Get the TIM address + 4...  Remember we are using a LONG pointer
  // so an increment of +1 with actually move the pointer 4 bytes....
  
  imgloc +=1;

	GsGetTimInfo(imgloc,&timdata);

	rect.x=timdata.px;
	rect.y=timdata.py;
	rect.w=timdata.pw;
	rect.h=timdata.ph;

	printf("RECT x = %d \n",rect.x);
	printf("RECT y = %d \n",rect.y);
	printf("RECT w = %d \n",rect.w);
	printf("RECT h = %d \n",rect.h);

	LoadImage(&rect,timdata.pixel);
	
	oursprite.attribute=0x02000000;
	oursprite.x=0;
	oursprite.y=0;
	oursprite.w=timdata.pw;
	oursprite.h=timdata.ph;
	oursprite.tpage=GetTPage(2,0,timdata.px,timdata.py);
	oursprite.u=0;
	oursprite.v=0;
	oursprite.r=oursprite.g=oursprite.b=128;
	oursprite.mx=oursprite.my=0;
	oursprite.scalex=oursprite.scaley=ONE;
	oursprite.rotate=0;
	DrawSync(0);
}

u_long PadRead(void)
{
	return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}

